@@ -243,8 +243,10 @@ entry_activate_cb (GtkEntry *entry,
   gtk_dialog_response (GTK_DIALOG (self), GTK_RESPONSE_ACCEPT);
 }
 
-static gboolean
+static void
 account_chooser_filter (TpAccount *account,
+    EmpathyAccountChooserFilterResultCallback callback,
+    gpointer callback_data,
     gpointer user_data)
 {
   EmpathyContactSelectorDialog *self = user_data;
@@ -252,9 +254,13 @@ account_chooser_filter (TpAccount *account,
       EMPATHY_CONTACT_SELECTOR_DIALOG_GET_CLASS (self);
 
   if (class->account_filter == NULL)
-    return empathy_account_chooser_filter_is_connected (account, user_data);
+    {
+      empathy_account_chooser_filter_is_connected (
+          account,callback, callback_data, user_data);
+      return;
+    }
 
-  return class->account_filter (self, account);
+  class->account_filter (self, callback, callback_data, account);
 }
 
 static gboolean
@@ -316,8 +322,6 @@ empathy_contact_selector_dialog_init (EmpathyContactSelectorDialog *dialog)
     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 
   /* Tweak the dialog */
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
@@ -492,7 +496,8 @@ empathy_contact_selector_dialog_class_init (
 const gchar *
 empathy_contact_selector_dialog_get_selected (
     EmpathyContactSelectorDialog *self,
-    TpConnection **connection)
+    TpConnection **connection,
+    TpAccount **account)
 {
   EmpathyContactSelectorDialogPriv *priv;
   const char *id;
@@ -501,7 +506,7 @@ empathy_contact_selector_dialog_get_selected (
 
   priv = GET_PRIV (self);
 
-  if (connection)
+  if (connection != NULL)
     {
       if (priv->show_account_chooser)
         *connection = empathy_account_chooser_get_connection (
@@ -510,6 +515,16 @@ empathy_contact_selector_dialog_get_selected (
         *connection = NULL;
     }
 
+  if (account != NULL)
+    {
+      if (priv->show_account_chooser)
+        *account = empathy_account_chooser_get_account (
+            EMPATHY_ACCOUNT_CHOOSER (priv->account_chooser));
+      else
+        *account = NULL;
+    }
+
+
   id = gtk_entry_get_text (GTK_ENTRY (priv->entry_id));
   return id;
 }
