@@ -379,6 +379,8 @@ create_errors_to_message_hash (void)
 		_("The length of the server certificate, or the depth of the "
 		"server certificate chain, exceed the limits imposed by the "
 		"cryptography library"));
+	g_hash_table_insert (errors, TP_ERROR_STR_SOFTWARE_UPGRADE_REQUIRED,
+		_("Your software is too old"));
 
 	return errors;
 }
@@ -738,7 +740,7 @@ empathy_contact_dup_from_folks_individual (FolksIndividual *individual)
 
 TpChannelGroupChangeReason
 tp_channel_group_change_reason_from_folks_groups_change_reason (
-    FolksGroupableChangeReason reason)
+    FolksGroupDetailsChangeReason reason)
 {
   return (TpChannelGroupChangeReason) reason;
 }
