@@ -1,6 +1,6 @@
 /*
  * empathy-call-window.c - Source for EmpathyCallWindow
- * Copyright (C) 2008-2009 Collabora Ltd.
+ * Copyright (C) 2008-2011 Collabora Ltd.
  * @author Sjoerd Simons <sjoerd.simons@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
@@ -30,19 +30,15 @@
 #include <glib/gi18n.h>
 
 #include <telepathy-glib/util.h>
-#include <telepathy-farsight/channel.h>
+#include <telepathy-farstream/telepathy-farstream.h>
 #include <telepathy-glib/util.h>
 
 #include <gst/farsight/fs-element-added-notifier.h>
+#include <gst/farsight/fs-utils.h>
 
 #include <libempathy/empathy-tp-contact-factory.h>
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy-gtk/empathy-avatar-image.h>
-#include <libempathy-gtk/empathy-video-widget.h>
-#include <libempathy-gtk/empathy-audio-src.h>
-#include <libempathy-gtk/empathy-audio-sink.h>
-#include <libempathy-gtk/empathy-video-src.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-sound.h>
 #include <libempathy-gtk/empathy-geometry.h>
@@ -53,6 +49,11 @@
 
 #include "empathy-call-window.h"
 #include "empathy-call-window-fullscreen.h"
+#include "empathy-call-factory.h"
+#include "empathy-video-widget.h"
+#include "empathy-audio-src.h"
+#include "empathy-audio-sink.h"
+#include "empathy-video-src.h"
 #include "empathy-sidebar.h"
 
 #define BUTTON_ID "empathy-call-dtmf-button-id"
@@ -78,16 +79,6 @@
 
 G_DEFINE_TYPE(EmpathyCallWindow, empathy_call_window, GTK_TYPE_WINDOW)
 
-/* signal enum */
-#if 0
-enum
-{
-    LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = {0};
-#endif
-
 enum {
   PROP_CALL_HANDLER = 1,
 };
@@ -105,13 +96,11 @@ typedef enum {
   CAMERA_STATE_ON,
 } CameraState;
 
-/* private structure */
-typedef struct _EmpathyCallWindowPriv EmpathyCallWindowPriv;
-
 struct _EmpathyCallWindowPriv
 {
   gboolean dispose_has_run;
   EmpathyCallHandler *handler;
+
   EmpathyContact *contact;
 
   guint call_state;
@@ -135,8 +124,7 @@ struct _EmpathyCallWindowPriv
   GtkWidget *pane;
   GtkAction *redial;
   GtkAction *menu_fullscreen;
-  GtkAction *action_camera;
-  GtkAction *action_camera_preview;
+  GtkAction *action_camera_on;
   GtkWidget *tool_button_camera_off;
   GtkWidget *tool_button_camera_preview;
   GtkWidget *tool_button_camera_on;
@@ -190,9 +178,8 @@ struct _EmpathyCallWindowPriv
   GstElement *video_tee;
 
   GstElement *funnel;
-  GstElement *liveadder;
 
-  FsElementAddedNotifier *fsnotifier;
+  GList *notifiers;
 
   guint context_id;
 
@@ -223,9 +210,7 @@ struct _EmpathyCallWindowPriv
   gboolean pipeline_playing;
 };
 
-#define GET_PRIV(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), EMPATHY_TYPE_CALL_WINDOW, \
-    EmpathyCallWindowPriv))
+#define GET_PRIV(o) (EMPATHY_CALL_WINDOW (o)->priv)
 
 static void empathy_call_window_realized_cb (GtkWidget *widget,
   EmpathyCallWindow *window);
@@ -279,9 +264,6 @@ static void empathy_call_window_restart_call (EmpathyCallWindow *window);
 static void empathy_call_window_status_message (EmpathyCallWindow *window,
   gchar *message);
 
-static void empathy_call_window_update_avatars_visibility (EmpathyTpCall *call,
-  EmpathyCallWindow *window);
-
 static gboolean empathy_call_window_bus_message (GstBus *bus,
   GstMessage *message, gpointer user_data);
 
@@ -341,17 +323,17 @@ static void
 dtmf_button_pressed_cb (GtkButton *button, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
   GQuark button_quark;
   TpDTMFEvent event;
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
 
   button_quark = g_quark_from_static_string (BUTTON_ID);
   event = GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (button),
     button_quark));
 
-  empathy_tp_call_start_tone (call, event);
+  tpy_call_channel_dtmf_start_tone (call, event);
 
   g_object_unref (call);
 }
@@ -360,11 +342,11 @@ static void
 dtmf_button_released_cb (GtkButton *button, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
 
-  empathy_tp_call_stop_tone (call);
+  tpy_call_channel_dtmf_stop_tone (call);
 
   g_object_unref (call);
 }
@@ -603,17 +585,9 @@ empathy_call_window_create_audio_input (EmpathyCallWindow *self)
 
   priv->volume_progress_bar = gtk_progress_bar_new ();
 
-  /* FIXME: Once we unconditionally depend on GTK+ 3, we can remove the fallback
-   * for GTK+ 2 here. GtkProgressBar doesn't implement GtkOrientable in GTK+ 2,
-   * unfortunately. */
-#ifndef HAVE_GTK3
   gtk_progress_bar_set_orientation (
-      GTK_PROGRESS_BAR (priv->volume_progress_bar),
+    GTK_PROGRESS_BAR (priv->volume_progress_bar),
       GTK_PROGRESS_BOTTOM_TO_TOP);
-#else
-  gtk_orientable_set_orientation (GTK_ORIENTABLE (priv->volume_progress_bar),
-      GTK_ORIENTATION_VERTICAL);
-#endif
 
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (priv->volume_progress_bar),
       0);
@@ -647,17 +621,6 @@ create_video_output_widget (EmpathyCallWindow *self)
   g_object_unref (bus);
 }
 
-static void
-create_audio_output (EmpathyCallWindow *self)
-{
-  EmpathyCallWindowPriv *priv = GET_PRIV (self);
-
-  g_assert (priv->audio_output == NULL);
-  priv->audio_output = empathy_audio_sink_new ();
-  gst_object_ref (priv->audio_output);
-  gst_object_sink (priv->audio_output);
-}
-
 static void
 create_video_input (EmpathyCallWindow *self)
 {
@@ -846,7 +809,7 @@ disable_camera (EmpathyCallWindow *self)
 
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (
       priv->tool_button_camera_off), TRUE);
-  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera),
+  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera_on),
       CAMERA_STATE_OFF);
   unblock_camera_control_signals (self);
 
@@ -903,7 +866,7 @@ enable_preview (EmpathyCallWindow *self)
 
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (
         priv->tool_button_camera_preview), TRUE);
-  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera),
+  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera_on),
       CAMERA_STATE_PREVIEW);
   unblock_camera_control_signals (self);
 
@@ -959,7 +922,7 @@ enable_camera (EmpathyCallWindow *self)
 
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (
       priv->tool_button_camera_on), TRUE);
-  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera),
+  gtk_radio_action_set_current_value (GTK_RADIO_ACTION (priv->action_camera_on),
       CAMERA_STATE_ON);
   unblock_camera_control_signals (self);
 
@@ -1038,15 +1001,17 @@ create_pipeline (EmpathyCallWindow *self)
 static void
 empathy_call_window_init (EmpathyCallWindow *self)
 {
-  EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  EmpathyCallWindowPriv *priv;
   GtkBuilder *gui;
   GtkWidget *top_vbox;
   GtkWidget *h;
   GtkWidget *arrow;
   GtkWidget *page;
   gchar *filename;
-  GKeyFile *keyfile;
-  GError *error = NULL;
+  GtkWidget *scroll;
+
+  priv = self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+    EMPATHY_TYPE_CALL_WINDOW, EmpathyCallWindowPriv);
 
   filename = empathy_file_lookup ("empathy-call-window.ui", "src");
   gui = empathy_builder_get_file (filename,
@@ -1063,8 +1028,7 @@ empathy_call_window_init (EmpathyCallWindow *self)
     "camera_off", &priv->tool_button_camera_off,
     "camera_preview", &priv->tool_button_camera_preview,
     "camera_on", &priv->tool_button_camera_on,
-    "action_camera_off",  &priv->action_camera,
-    "action_camera_preview",  &priv->action_camera_preview,
+    "action_camera_on",  &priv->action_camera_on,
     "details_vbox",  &priv->details_vbox,
     "vcodec_encoding_label", &priv->vcodec_encoding_label,
     "acodec_encoding_label", &priv->acodec_encoding_label,
@@ -1091,7 +1055,7 @@ empathy_call_window_init (EmpathyCallWindow *self)
     "camera_off", "toggled", tool_button_camera_off_toggled_cb,
     "camera_preview", "toggled", tool_button_camera_preview_toggled_cb,
     "camera_on", "toggled", tool_button_camera_on_toggled_cb,
-    "action_camera_off", "changed", action_camera_change_cb,
+    "action_camera_on", "changed", action_camera_change_cb,
     NULL);
 
   gtk_action_set_sensitive (priv->menu_fullscreen, FALSE);
@@ -1140,30 +1104,11 @@ empathy_call_window_init (EmpathyCallWindow *self)
   create_pipeline (self);
   create_video_output_widget (self);
   create_audio_input (self);
-  create_audio_output (self);
   create_video_input (self);
 
-  priv->fsnotifier = fs_element_added_notifier_new ();
-  fs_element_added_notifier_add (priv->fsnotifier, GST_BIN (priv->pipeline));
-
   /* The call will be started as soon the pipeline is playing */
   priv->start_call_when_playing = TRUE;
 
-  keyfile = g_key_file_new ();
-  filename = empathy_file_lookup ("element-properties", "data");
-  if (g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, &error))
-    {
-      fs_element_added_notifier_set_properties_from_keyfile (priv->fsnotifier,
-          keyfile);
-    }
-  else
-    {
-      g_warning ("Could not load element-properties file: %s", error->message);
-      g_key_file_free (keyfile);
-      g_clear_error (&error);
-    }
-  g_free (filename);
-
   priv->vbox = gtk_vbox_new (FALSE, 3);
   gtk_box_pack_start (GTK_BOX (priv->content_hbox), priv->vbox,
       FALSE, FALSE, CONTENT_HBOX_CHILDREN_PACKING_PADDING);
@@ -1191,21 +1136,27 @@ empathy_call_window_init (EmpathyCallWindow *self)
   gtk_paned_pack2 (GTK_PANED (priv->pane), priv->sidebar, FALSE, FALSE);
 
   page = empathy_call_window_create_audio_input (self);
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar), _("Audio input"),
-    page);
+  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+      _("Audio input"), page);
 
   page = empathy_call_window_create_video_input (self);
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar), _("Video input"),
-    page);
+  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+      _("Video input"), page);
 
   priv->dtmf_panel = empathy_call_window_create_dtmf (self);
-  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar), _("Dialpad"),
-    priv->dtmf_panel);
+  empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar),
+      _("Dialpad"), priv->dtmf_panel);
 
   gtk_widget_set_sensitive (priv->dtmf_panel, FALSE);
 
+  /* Put the details vbox in a scroll window as it can require a lot of
+   * horizontal space. */
+  scroll = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scroll),
+      priv->details_vbox);
+
   empathy_sidebar_add_page (EMPATHY_SIDEBAR (priv->sidebar), _("Details"),
-    priv->details_vbox);
+    scroll);
 
   gtk_widget_show_all (top_vbox);
 
@@ -1270,12 +1221,19 @@ set_window_title (EmpathyCallWindow *self)
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
   gchar *tmp;
 
-  /* translators: Call is a noun and %s is the contact name. This string
-   * is used in the window title */
-  tmp = g_strdup_printf (_("Call with %s"),
-      empathy_contact_get_alias (priv->contact));
-  gtk_window_set_title (GTK_WINDOW (self), tmp);
-  g_free (tmp);
+  if (priv->contact != NULL)
+    {
+      /* translators: Call is a noun and %s is the contact name. This string
+       * is used in the window title */
+      tmp = g_strdup_printf (_("Call with %s"),
+          empathy_contact_get_alias (priv->contact));
+      gtk_window_set_title (GTK_WINDOW (self), tmp);
+      g_free (tmp);
+    }
+  else
+    {
+      gtk_window_set_title (GTK_WINDOW (self), _("Call with %d participants"));
+    }
 }
 
 static void
@@ -1325,40 +1283,22 @@ empathy_call_window_setup_avatars (EmpathyCallWindow *self,
     EmpathyCallHandler *handler)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  TpConnection *connection;
 
-  g_object_get (handler, "contact", &(priv->contact), NULL);
+  g_signal_connect (priv->contact, "notify::name",
+      G_CALLBACK (contact_name_changed_cb), self);
+  g_signal_connect (priv->contact, "notify::avatar",
+    G_CALLBACK (contact_avatar_changed_cb),
+    priv->remote_user_avatar_widget);
 
-  if (priv->contact != NULL)
-    {
-      TpConnection *connection;
+  /* Retrieving the self avatar */
+  connection = empathy_contact_get_connection (priv->contact);
+  empathy_tp_contact_factory_get_from_handle (connection,
+      tp_connection_get_self_handle (connection),
+      empathy_call_window_got_self_contact_cb, self, NULL,
+      G_OBJECT (self));
 
-      set_window_title (self);
-
-      g_signal_connect (priv->contact, "notify::name",
-          G_CALLBACK (contact_name_changed_cb), self);
-      g_signal_connect (priv->contact, "notify::avatar",
-          G_CALLBACK (contact_avatar_changed_cb),
-          priv->remote_user_avatar_widget);
-
-      /* Retreiving the self avatar */
-      connection = empathy_contact_get_connection (priv->contact);
-      empathy_tp_contact_factory_get_from_handle (connection,
-          tp_connection_get_self_handle (connection),
-          empathy_call_window_got_self_contact_cb, self, NULL, G_OBJECT (self));
-    }
-  else
-    {
-      g_warning ("call handler doesn't have a contact");
-      /* translators: Call is a noun. This string is used in the window
-       * title */
-      gtk_window_set_title (GTK_WINDOW (self), _("Call"));
-
-      /* Since we can't access the remote contact, we can't get a connection
-         to it and can't get the self contact (and its avatar). This means
-         that we have to manually set the self avatar. */
-      init_contact_avatar_with_size (NULL, priv->self_user_avatar_widget,
-          MIN (SELF_VIDEO_SECTION_WIDTH, SELF_VIDEO_SECTION_HEIGTH));
-    }
+  set_window_title (self);
 
   init_contact_avatar_with_size (priv->contact,
       priv->remote_user_avatar_widget,
@@ -1584,15 +1524,18 @@ empathy_call_window_constructed (GObject *object)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (object);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
 
   g_assert (priv->handler != NULL);
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
   priv->outgoing = (call == NULL);
   if (call != NULL)
     g_object_unref (call);
 
+  g_object_get (priv->handler, "target-contact", &priv->contact, NULL);
+  g_assert (priv->contact != NULL);
+
   empathy_call_window_setup_avatars (self, priv->handler);
   empathy_call_window_set_state_connecting (self);
 
@@ -1682,18 +1625,9 @@ empathy_call_window_class_init (
     PROP_CALL_HANDLER, param_spec);
 }
 
-static void
-empathy_call_window_video_stream_changed_cb (EmpathyTpCall *call,
-    GParamSpec *property, EmpathyCallWindow *self)
-{
-  DEBUG ("video stream changed");
-  empathy_call_window_update_avatars_visibility (call, self);
-}
-
 void
 empathy_call_window_dispose (GObject *object)
 {
-  EmpathyTpCall *call;
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (object);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
 
@@ -1702,19 +1636,11 @@ empathy_call_window_dispose (GObject *object)
 
   priv->dispose_has_run = TRUE;
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
-
-  if (call != NULL)
-    {
-      g_object_unref (call);
-    }
-
   if (priv->handler != NULL)
     {
       empathy_call_handler_stop_call (priv->handler);
-      g_object_unref (priv->handler);
+      tp_clear_object (&priv->handler);
     }
-  priv->handler = NULL;
 
   if (priv->bus_message_source_id != 0)
     {
@@ -1722,57 +1648,28 @@ empathy_call_window_dispose (GObject *object)
       priv->bus_message_source_id = 0;
     }
 
-  if (priv->pipeline != NULL)
-    g_object_unref (priv->pipeline);
-  priv->pipeline = NULL;
+  tp_clear_object (&priv->pipeline);
+  tp_clear_object (&priv->video_input);
+  tp_clear_object (&priv->audio_input);
+  tp_clear_object (&priv->video_tee);
+  tp_clear_object (&priv->ui_manager);
+  tp_clear_object (&priv->fullscreen);
 
-  if (priv->video_input != NULL)
-    g_object_unref (priv->video_input);
-  priv->video_input = NULL;
-
-  if (priv->audio_input != NULL)
-    g_object_unref (priv->audio_input);
-  priv->audio_input = NULL;
-
-  if (priv->audio_output != NULL)
-    g_object_unref (priv->audio_output);
-  priv->audio_output = NULL;
-
-  if (priv->video_tee != NULL)
-    g_object_unref (priv->video_tee);
-  priv->video_tee = NULL;
-
-  if (priv->liveadder != NULL)
-    gst_object_unref (priv->liveadder);
-  priv->liveadder = NULL;
-
-  if (priv->fsnotifier != NULL)
-    g_object_unref (priv->fsnotifier);
-  priv->fsnotifier = NULL;
+  g_list_free_full (priv->notifiers, g_object_unref);
 
   if (priv->timer_id != 0)
     g_source_remove (priv->timer_id);
   priv->timer_id = 0;
 
-  if (priv->ui_manager != NULL)
-    g_object_unref (priv->ui_manager);
-  priv->ui_manager = NULL;
-
-  if (priv->fullscreen != NULL)
-    g_object_unref (priv->fullscreen);
-  priv->fullscreen = NULL;
-
   if (priv->contact != NULL)
     {
       g_signal_handlers_disconnect_by_func (priv->contact,
           contact_name_changed_cb, self);
-      g_object_unref (priv->contact);
       priv->contact = NULL;
     }
 
-  /* release any references held by the object here */
-  if (G_OBJECT_CLASS (empathy_call_window_parent_class)->dispose)
-    G_OBJECT_CLASS (empathy_call_window_parent_class)->dispose (object);
+
+  G_OBJECT_CLASS (empathy_call_window_parent_class)->dispose (object);
 }
 
 static void
@@ -1818,27 +1715,35 @@ empathy_call_window_conference_added_cb (EmpathyCallHandler *handler,
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  FsElementAddedNotifier *notifier;
+  GKeyFile *keyfile;
+
+  g_debug ("Conference added");
+
+  /* Add notifier to set the various element properties as needed */
+  notifier = fs_element_added_notifier_new ();
+  keyfile = fs_utils_get_default_element_properties (conference);
+
+  if (keyfile != NULL)
+    fs_element_added_notifier_set_properties_from_keyfile (notifier, keyfile);
+
+  fs_element_added_notifier_add (notifier, GST_BIN (priv->pipeline));
+
+  priv->notifiers = g_list_prepend (priv->notifiers, notifier);
 
   gst_bin_add (GST_BIN (priv->pipeline), conference);
-
   gst_element_set_state (conference, GST_STATE_PLAYING);
 }
 
-static gboolean
-empathy_call_window_request_resource_cb (EmpathyCallHandler *handler,
-  FsMediaType type, FsStreamDirection direction, gpointer user_data)
+static void
+empathy_call_window_conference_removed_cb (EmpathyCallHandler *handler,
+  GstElement *conference, gpointer user_data)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
 
-  if (type != FS_MEDIA_TYPE_VIDEO)
-    return TRUE;
-
-  if (direction == FS_DIRECTION_RECV)
-    return TRUE;
-
-  /* video and direction is send */
-  return priv->video_input != NULL;
+  gst_bin_remove (GST_BIN (priv->pipeline), conference);
+  gst_element_set_state (conference, GST_STATE_NULL);
 }
 
 static gboolean
@@ -1876,7 +1781,6 @@ empathy_call_window_reset_pipeline (EmpathyCallWindow *self)
         gtk_widget_destroy (priv->video_preview);
       priv->video_preview = NULL;
 
-      priv->liveadder = NULL;
       priv->funnel = NULL;
 
       create_pipeline (self);
@@ -1951,6 +1855,7 @@ empathy_call_window_disconnected (EmpathyCallWindow *self,
 
       /* Unsensitive the camera and mic button */
       gtk_widget_set_sensitive (priv->tool_button_camera_on, FALSE);
+      gtk_action_set_sensitive (priv->action_camera_on, FALSE);
       gtk_widget_set_sensitive (priv->mic_button, FALSE);
 
       /* Be sure that the mic button is enabled */
@@ -1974,7 +1879,8 @@ empathy_call_window_disconnected (EmpathyCallWindow *self,
 
       /* destroy the video output; it will be recreated when we'll redial */
       disconnect_video_output_motion_handler (self);
-      gtk_widget_destroy (priv->video_output);
+      if (priv->video_output != NULL)
+        gtk_widget_destroy (priv->video_output);
       priv->video_output = NULL;
 
       gtk_widget_show (priv->remote_user_avatar_widget);
@@ -2006,22 +1912,21 @@ empathy_call_window_channel_closed_cb (EmpathyCallHandler *handler,
       empathy_call_window_restart_call (self);
 }
 
-
-static void
-empathy_call_window_channel_stream_closed_cb (EmpathyCallHandler *handler,
-    TfStream *stream, gpointer user_data)
+static gboolean
+empathy_call_window_sink_removed_cb (EmpathyCallHandler *handler,
+    GstPad *sink,
+    FsMediaType media_type,
+    EmpathyCallWindow *self)
 {
-  EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  guint media_type;
 
-  g_object_get (stream, "media-type", &media_type, NULL);
+  g_print ("window: removing content\n");
 
   /*
    * This assumes that there is only one video stream per channel...
    */
 
-  if (media_type == TP_MEDIA_STREAM_TYPE_VIDEO)
+  if (media_type == FS_MEDIA_TYPE_VIDEO)
     {
       if (priv->funnel != NULL)
         {
@@ -2036,20 +1941,22 @@ empathy_call_window_channel_stream_closed_cb (EmpathyCallHandler *handler,
           gst_bin_remove (GST_BIN (priv->pipeline), output);
           gst_bin_remove (GST_BIN (priv->pipeline), priv->funnel);
           priv->funnel = NULL;
+          return TRUE;
         }
     }
-  else if (media_type == TP_MEDIA_STREAM_TYPE_AUDIO)
+  else if (media_type == FS_MEDIA_TYPE_AUDIO)
     {
-      if (priv->liveadder != NULL)
+      if (priv->audio_output != NULL)
         {
           gst_element_set_state (priv->audio_output, GST_STATE_NULL);
-          gst_element_set_state (priv->liveadder, GST_STATE_NULL);
 
           gst_bin_remove (GST_BIN (priv->pipeline), priv->audio_output);
-          gst_bin_remove (GST_BIN (priv->pipeline), priv->liveadder);
-          priv->liveadder = NULL;
+          priv->audio_output = NULL;
+          return TRUE;
         }
     }
+
+  return FALSE;
 }
 
 /* Called with global lock held */
@@ -2139,112 +2046,47 @@ empathy_call_window_get_audio_sink_pad (EmpathyCallWindow *self)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
   GstPad *pad;
-  GstElement *filter;
-  GError *gerror = NULL;
+  GstPadTemplate *template;
 
-  if (priv->liveadder == NULL)
+  if (priv->audio_output == NULL)
     {
-      priv->liveadder = gst_element_factory_make ("liveadder", NULL);
+      priv->audio_output = empathy_audio_sink_new ();
 
-      if (!gst_bin_add (GST_BIN (priv->pipeline), priv->liveadder))
-        {
-          g_warning ("Could not add liveadder to the pipeline");
-          goto error_add_liveadder;
-        }
       if (!gst_bin_add (GST_BIN (priv->pipeline), priv->audio_output))
         {
           g_warning ("Could not add audio sink to pipeline");
+          g_object_unref (priv->audio_output);
           goto error_add_output;
         }
 
-      if (gst_element_set_state (priv->liveadder, GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE)
-        {
-          g_warning ("Could not start liveadder");
-          goto error;
-        }
-
       if (gst_element_set_state (priv->audio_output, GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE)
         {
           g_warning ("Could not start audio sink");
           goto error;
         }
-
-      if (GST_PAD_LINK_FAILED (
-              gst_element_link (priv->liveadder, priv->audio_output)))
-        {
-          g_warning ("Could not link liveadder to audio output");
-          goto error;
-        }
     }
 
-  filter = gst_parse_bin_from_description (
-      "audioconvert ! audioresample ! audioconvert", TRUE, &gerror);
-  if (filter == NULL)
-    {
-      g_warning ("Could not make audio conversion filter: %s", gerror->message);
-      g_clear_error (&gerror);
-      goto error;
-    }
+  template = gst_element_class_get_pad_template (
+    GST_ELEMENT_GET_CLASS (priv->audio_output), "sink%d");
 
-  if (!gst_bin_add (GST_BIN (priv->pipeline), filter))
-    {
-      g_warning ("Could not add audio conversion filter to pipeline");
-      gst_object_unref (filter);
-      goto error;
-    }
-
-  if (gst_element_set_state (filter, GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE)
-    {
-      g_warning ("Could not start audio conversion filter");
-      goto error_filter;
-    }
-
-  if (!gst_element_link (filter, priv->liveadder))
-    {
-      g_warning ("Could not link audio conversion filter to liveadder");
-      goto error_filter;
-    }
-
-  pad = gst_element_get_static_pad (filter, "sink");
+  pad = gst_element_request_pad (priv->audio_output,
+    template, NULL, NULL);
 
   if (pad == NULL)
     {
-      g_warning ("Could not get sink pad from filter");
-      goto error_filter;
+      g_warning ("Could not get sink pad from sink");
+      return NULL;
     }
 
   return pad;
 
- error_filter:
-
-  gst_element_set_locked_state (filter, TRUE);
-  gst_element_set_state (filter, GST_STATE_NULL);
-  gst_bin_remove (GST_BIN (priv->pipeline), filter);
-
- error:
-
-  gst_element_set_locked_state (priv->liveadder, TRUE);
+error:
   gst_element_set_locked_state (priv->audio_output, TRUE);
-
-  gst_element_set_state (priv->liveadder, GST_STATE_NULL);
   gst_element_set_state (priv->audio_output, GST_STATE_NULL);
-
   gst_bin_remove (GST_BIN (priv->pipeline), priv->audio_output);
+  priv->audio_output = NULL;
 
- error_add_output:
-
-  gst_bin_remove (GST_BIN (priv->pipeline), priv->liveadder);
-
-  gst_element_set_locked_state (priv->liveadder, FALSE);
-  gst_element_set_locked_state (priv->audio_output, FALSE);
-
- error_add_liveadder:
-
-  if (priv->liveadder != NULL)
-    {
-      gst_object_unref (priv->liveadder);
-      priv->liveadder = NULL;
-    }
+error_add_output:
 
   return NULL;
 }
@@ -2268,9 +2110,10 @@ empathy_call_window_update_timer (gpointer user_data)
   return TRUE;
 }
 
+#if 0
 static void
 display_error (EmpathyCallWindow *self,
-    EmpathyTpCall *call,
+    TpyCallChannel *call,
     const gchar *img,
     const gchar *title,
     const gchar *desc,
@@ -2345,12 +2188,12 @@ display_error (EmpathyCallWindow *self,
 
 static gchar *
 media_stream_error_to_txt (EmpathyCallWindow *self,
-    EmpathyTpCall *call,
+    TpyCallChannel *call,
     gboolean audio,
     TpMediaStreamError error)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  const gchar *cm;
+  const gchar *cm = NULL;
   gchar *url;
   gchar *result;
 
@@ -2387,7 +2230,9 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
                 "are not installed on your computer"));
 
       case TP_MEDIA_STREAM_ERROR_INVALID_CM_BEHAVIOR:
-        cm = empathy_tp_call_get_connection_manager (call);
+        tp_connection_parse_object_path (
+            tp_channel_borrow_connection (TP_CHANNEL (call)),
+            NULL, &cm);
 
         url = g_strdup_printf ("http://bugs.freedesktop.org/enter_bug.cgi?"
             "product=Telepathy&amp;component=%s", cm);
@@ -2398,6 +2243,7 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
               "logs gathered from the 'Debug' window in the Help menu."), url);
 
         g_free (url);
+        g_free (cm);
         return result;
 
       case TP_MEDIA_STREAM_ERROR_MEDIA_ERROR:
@@ -2414,7 +2260,7 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
 
 static void
 empathy_call_window_stream_error (EmpathyCallWindow *self,
-    EmpathyTpCall *call,
+    TpyCallChannel *call,
     gboolean audio,
     guint code,
     const gchar *msg,
@@ -2438,7 +2284,7 @@ empathy_call_window_stream_error (EmpathyCallWindow *self,
 }
 
 static void
-empathy_call_window_audio_stream_error (EmpathyTpCall *call,
+empathy_call_window_audio_stream_error (TpyCallChannel *call,
     guint code,
     const gchar *msg,
     EmpathyCallWindow *self)
@@ -2448,7 +2294,7 @@ empathy_call_window_audio_stream_error (EmpathyTpCall *call,
 }
 
 static void
-empathy_call_window_video_stream_error (EmpathyTpCall *call,
+empathy_call_window_video_stream_error (TpyCallChannel *call,
     guint code,
     const gchar *msg,
     EmpathyCallWindow *self)
@@ -2456,46 +2302,55 @@ empathy_call_window_video_stream_error (EmpathyTpCall *call,
   empathy_call_window_stream_error (self, call, FALSE, code, msg,
       "camera-web", _("Can't establish video stream"));
 }
+#endif
 
-static gboolean
-empathy_call_window_connected (gpointer user_data)
+static void
+empathy_call_window_state_changed_cb (EmpathyCallHandler *handler,
+    TpyCallState state,
+    EmpathyCallWindow *self)
 {
-  EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
   gboolean can_send_video;
 
+  if (state != TPY_CALL_STATE_ACCEPTED)
+    return;
+
+  if (priv->call_state == CONNECTED)
+    return;
+
+  g_timer_start (priv->timer);
+  priv->call_state = CONNECTED;
+
   empathy_sound_stop (EMPATHY_SOUND_PHONE_OUTGOING);
 
-  can_send_video = priv->video_input != NULL && priv->contact != NULL &&
+  can_send_video = priv->video_input != NULL &&
     empathy_contact_can_voip_video (priv->contact);
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
 
-  tp_g_signal_connect_object (call, "notify::video-stream",
-    G_CALLBACK (empathy_call_window_video_stream_changed_cb),
-    self, 0);
-
-  if (empathy_tp_call_has_dtmf (call))
+  if (tpy_call_channel_has_dtmf (call))
     gtk_widget_set_sensitive (priv->dtmf_panel, TRUE);
 
   if (priv->video_input == NULL)
     empathy_call_window_set_send_video (self, CAMERA_STATE_OFF);
 
-  priv->sending_video = can_send_video ?
-    empathy_tp_call_is_sending_video (call) : FALSE;
+  priv->sending_video = can_send_video &&
+    empathy_call_handler_has_initial_video (priv->handler);
 
   gtk_toggle_tool_button_set_active (
       GTK_TOGGLE_TOOL_BUTTON (priv->tool_button_camera_on),
       priv->sending_video && priv->video_input != NULL);
   gtk_widget_set_sensitive (priv->tool_button_camera_on, can_send_video);
+  gtk_action_set_sensitive (priv->action_camera_on, can_send_video);
 
   gtk_action_set_sensitive (priv->redial, FALSE);
   gtk_widget_set_sensitive (priv->redial_button, FALSE);
 
   gtk_widget_set_sensitive (priv->mic_button, TRUE);
 
-  empathy_call_window_update_avatars_visibility (call, self);
+  gtk_widget_hide (priv->video_output);
+  gtk_widget_show (priv->remote_user_avatar_widget);
 
   g_object_unref (call);
 
@@ -2509,11 +2364,19 @@ empathy_call_window_connected (gpointer user_data)
   empathy_call_window_update_timer (self);
 
   gtk_action_set_sensitive (priv->menu_fullscreen, TRUE);
+}
+
+static gboolean
+emapthy_call_window_show_video_output_cb (gpointer user_data)
+{
+  EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
+
+  gtk_widget_hide (self->priv->remote_user_avatar_widget);
+  gtk_widget_show (self->priv->video_output);
 
   return FALSE;
 }
 
-
 /* Called from the streaming thread */
 static gboolean
 empathy_call_window_src_added_cb (EmpathyCallHandler *handler,
@@ -2527,21 +2390,13 @@ empathy_call_window_src_added_cb (EmpathyCallHandler *handler,
 
   g_mutex_lock (priv->lock);
 
-  if (priv->call_state != CONNECTED)
-    {
-      g_timer_start (priv->timer);
-      priv->timer_id = g_idle_add  (empathy_call_window_connected, self);
-      priv->call_state = CONNECTED;
-    }
-
   switch (media_type)
     {
       case TP_MEDIA_STREAM_TYPE_AUDIO:
         pad = empathy_call_window_get_audio_sink_pad (self);
         break;
       case TP_MEDIA_STREAM_TYPE_VIDEO:
-        gtk_widget_hide (priv->remote_user_avatar_widget);
-        gtk_widget_show (priv->video_output);
+        g_idle_add (emapthy_call_window_show_video_output_cb, self);
         pad = empathy_call_window_get_video_sink_pad (self);
         break;
       default:
@@ -2598,7 +2453,7 @@ empathy_call_window_src_added_cb (EmpathyCallHandler *handler,
 
 static gboolean
 empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
-  GstPad *sink, guint media_type, gpointer user_data)
+  GstPad *sink, FsMediaType media_type, gpointer user_data)
 {
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
@@ -2607,7 +2462,7 @@ empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
 
   switch (media_type)
     {
-      case TP_MEDIA_STREAM_TYPE_AUDIO:
+      case FS_MEDIA_TYPE_AUDIO:
         if (!gst_bin_add (GST_BIN (priv->pipeline), priv->audio_input))
           {
             g_warning ("Could not add audio source to pipeline");
@@ -2639,7 +2494,7 @@ empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
 
         retval = TRUE;
         break;
-      case TP_MEDIA_STREAM_TYPE_VIDEO:
+      case FS_MEDIA_TYPE_VIDEO:
         if (priv->video_input != NULL)
           {
             if (priv->video_tee != NULL)
@@ -2647,7 +2502,7 @@ empathy_call_window_sink_added_cb (EmpathyCallHandler *handler,
                 pad = gst_element_get_request_pad (priv->video_tee, "src%d");
                 if (GST_PAD_LINK_FAILED (gst_pad_link (pad, sink)))
                   {
-                    g_warning ("Could not link videp soure input pipeline");
+                    g_warning ("Could not link video source input pipeline");
                     break;
                   }
                 gst_object_unref (pad);
@@ -2690,6 +2545,7 @@ empathy_call_window_remove_video_input (EmpathyCallWindow *self)
   priv->video_preview = NULL;
 
   gtk_widget_set_sensitive (priv->tool_button_camera_on, FALSE);
+  gtk_action_set_sensitive (priv->action_camera_on, FALSE);
   gtk_widget_set_sensitive (priv->tool_button_camera_preview, FALSE);
 }
 
@@ -2796,40 +2652,23 @@ empathy_call_window_bus_message (GstBus *bus, GstMessage *message,
 }
 
 static void
-empathy_call_window_update_avatars_visibility (EmpathyTpCall *call,
-    EmpathyCallWindow *window)
-{
-  EmpathyCallWindowPriv *priv = GET_PRIV (window);
-
-  if (empathy_tp_call_is_receiving_video (call))
-    {
-      gtk_widget_hide (priv->remote_user_avatar_widget);
-      gtk_widget_show (priv->video_output);
-    }
-  else
-    {
-      gtk_widget_hide (priv->video_output);
-      gtk_widget_show (priv->remote_user_avatar_widget);
-    }
-}
-
-static void
-call_handler_notify_tp_call_cb (EmpathyCallHandler *handler,
+call_handler_notify_call_cb (EmpathyCallHandler *handler,
     GParamSpec *spec,
     EmpathyCallWindow *self)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
   if (call == NULL)
     return;
 
+/* FIXME
   tp_g_signal_connect_object (call, "audio-stream-error",
       G_CALLBACK (empathy_call_window_audio_stream_error), self, 0);
   tp_g_signal_connect_object (call, "video-stream-error",
       G_CALLBACK (empathy_call_window_video_stream_error), self, 0);
-
+*/
   g_object_unref (call);
 }
 
@@ -2837,39 +2676,42 @@ static void
 empathy_call_window_realized_cb (GtkWidget *widget, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
 
+  g_signal_connect (priv->handler, "state-changed",
+    G_CALLBACK (empathy_call_window_state_changed_cb), window);
   g_signal_connect (priv->handler, "conference-added",
     G_CALLBACK (empathy_call_window_conference_added_cb), window);
-  g_signal_connect (priv->handler, "request-resource",
-    G_CALLBACK (empathy_call_window_request_resource_cb), window);
+  g_signal_connect (priv->handler, "conference-removed",
+    G_CALLBACK (empathy_call_window_conference_removed_cb), window);
   g_signal_connect (priv->handler, "closed",
     G_CALLBACK (empathy_call_window_channel_closed_cb), window);
   g_signal_connect (priv->handler, "src-pad-added",
     G_CALLBACK (empathy_call_window_src_added_cb), window);
   g_signal_connect (priv->handler, "sink-pad-added",
     G_CALLBACK (empathy_call_window_sink_added_cb), window);
-  g_signal_connect (priv->handler, "stream-closed",
-    G_CALLBACK (empathy_call_window_channel_stream_closed_cb), window);
+  g_signal_connect (priv->handler, "sink-pad-removed",
+    G_CALLBACK (empathy_call_window_sink_removed_cb), window);
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
   if (call != NULL)
     {
+/* FIXME: part of the improvements for DRAFT2
       tp_g_signal_connect_object (call, "audio-stream-error",
         G_CALLBACK (empathy_call_window_audio_stream_error), window,
         0);
       tp_g_signal_connect_object (call, "video-stream-error",
         G_CALLBACK (empathy_call_window_video_stream_error), window,
         0);
-
+*/
       g_object_unref (call);
     }
   else
     {
-      /* tp-call doesn't exist yet, we'll connect signals once it has been
+      /* call-channel doesn't exist yet, we'll connect signals once it has been
        * set */
-      g_signal_connect (priv->handler, "notify::tp-call",
-        G_CALLBACK (call_handler_notify_tp_call_cb), window);
+      g_signal_connect (priv->handler, "notify::call-channel",
+        G_CALLBACK (call_handler_notify_call_cb), window);
     }
 
   gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
@@ -3046,7 +2888,7 @@ empathy_call_window_set_send_video (EmpathyCallWindow *window,
   CameraState state)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
-  EmpathyTpCall *call;
+  TpyCallChannel *call;
 
   priv->sending_video = (state == CAMERA_STATE_ON);
 
@@ -3065,9 +2907,9 @@ empathy_call_window_set_send_video (EmpathyCallWindow *window,
   if (priv->call_state != CONNECTED)
     return;
 
-  g_object_get (priv->handler, "tp-call", &call, NULL);
+  g_object_get (priv->handler, "call-channel", &call, NULL);
   DEBUG ("%s sending video", priv->sending_video ? "start": "stop");
-  empathy_tp_call_request_video_stream_direction (call, priv->sending_video);
+  tpy_call_channel_send_video (call, priv->sending_video);
   g_object_unref (call);
 }
 
@@ -3218,7 +3060,7 @@ empathy_call_window_key_press_cb (GtkWidget *video_output,
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
 
-  if (priv->is_fullscreen && event->keyval == GDK_Escape)
+  if (priv->is_fullscreen && event->keyval == GDK_KEY_Escape)
     {
       /* Since we are in fullscreen mode, toggling will bring us back to
          normal mode. */
@@ -3304,7 +3146,7 @@ block_camera_control_signals (EmpathyCallWindow *self)
       tool_button_camera_preview_toggled_cb, self);
   g_signal_handlers_block_by_func (priv->tool_button_camera_on,
       tool_button_camera_on_toggled_cb, self);
-  g_signal_handlers_block_by_func (priv->action_camera,
+  g_signal_handlers_block_by_func (priv->action_camera_on,
       action_camera_change_cb, self);
 }
 
@@ -3319,6 +3161,6 @@ unblock_camera_control_signals (EmpathyCallWindow *self)
       tool_button_camera_preview_toggled_cb, self);
   g_signal_handlers_unblock_by_func (priv->tool_button_camera_on,
       tool_button_camera_on_toggled_cb, self);
-  g_signal_handlers_unblock_by_func (priv->action_camera,
+  g_signal_handlers_unblock_by_func (priv->action_camera_on,
       action_camera_change_cb, self);
 }
