@@ -403,6 +403,10 @@ chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
 
 	if (orig_submenu == NULL || !gtk_widget_get_visible (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
+
+		/* gtk_menu_attach_to_widget () doesn't behave nicely here */
+		g_object_set_data (G_OBJECT (submenu), "window", priv->dialog);
+
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
 		gtk_widget_show (menu);
 	} else {
