@@ -289,7 +289,7 @@ tpy_call_channel_dispose (GObject *obj)
   TpyCallChannel *self = (TpyCallChannel *) obj;
 
   tp_clear_pointer (&self->priv->contents, g_ptr_array_unref);
-  self->priv->contents = NULL;
+  tp_clear_pointer (&self->priv->details, g_hash_table_unref);
 
   tp_clear_object (&self->priv->result);
 
@@ -760,15 +760,38 @@ tpy_call_channel_hangup_finish (TpyCallChannel *self,
 {
 }
 
+TpyCallState
+tpy_call_channel_get_state (TpyCallChannel *self,
+    TpyCallFlags *flags, GHashTable **details)
+{
+  g_return_val_if_fail (TPY_IS_CALL_CHANNEL (self), TPY_CALL_STATE_UNKNOWN);
+
+  if (flags != NULL)
+    *flags = self->priv->flags;
+
+  if (details != NULL)
+    {
+      if (self->priv->details != NULL)
+        g_hash_table_ref (self->priv->details);
+
+      *details = self->priv->details;
+    }
+  return self->priv->state;
+}
+
 gboolean
 tpy_call_channel_has_initial_video (TpyCallChannel *self)
 {
+  g_return_val_if_fail (TPY_IS_CALL_CHANNEL (self), FALSE);
+
   return self->priv->initial_video;
 }
 
 gboolean
 tpy_call_channel_has_initial_audio (TpyCallChannel *self)
 {
+  g_return_val_if_fail (TPY_IS_CALL_CHANNEL (self), FALSE);
+
   return self->priv->initial_audio;
 }
 
@@ -779,6 +802,8 @@ tpy_call_channel_send_video (TpyCallChannel *self,
   gboolean found = FALSE;
   guint i;
 
+  g_return_if_fail (TPY_IS_CALL_CHANNEL (self));
+
   /* Loop over all the contents, if some of them a video set all their
    * streams to sending, otherwise request a video channel in case we want to
    * sent */
